-- This software is distributed under the terms of the new BSD License.
-- Copyright (c) 2014 molelord
-- All rights reserved.

-- チャタリングするスイッチ入力を、ディジタル的におとなしくさせる回路。


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity debounce_rst is
generic (
    INIT   : std_logic := '0';
    CLK_HZ : natural   := 10000000 -- クロック周波数
);
port (
    clk_i : in  std_logic;
    rst_i : in  std_logic;
    rst_o : out std_logic
);
end debounce_rst;

architecture rtl of debounce_rst is

    constant THRESHOLD : natural := 50; -- ミリ秒単位

    constant COUNT_MAX : natural := THRESHOLD * CLK_HZ / 1000 - 1;

    signal highcount : integer range 0 to COUNT_MAX;
    signal lowcount  : integer range 0 to COUNT_MAX;

    -- FlipFlopに初期値を与えているので、不本意ながらXilinx専用。
    signal current   : std_logic := INIT;
    signal rst       : std_logic := INIT;
    signal rst_ff    : std_logic := INIT;

begin

    -- 非同期入力なのでDouble flopsする。
    process (clk_i) begin
        if clk_i'event and clk_i = '1' then
            rst    <= rst_i;
            rst_ff <= rst;
        end if;
    end process;

    -- Highが続く間highcountをカウントアップ、
    -- Lowが続く間lowcountをカウントアップ、
    -- MAXに到達したら出力を変更する。
    process (clk_i) begin
        if clk_i'event and clk_i = '1' then
            if rst_ff = '1' then
                if highcount < COUNT_MAX then
                    highcount <= highcount + 1;
                end if;
                lowcount  <= 0;
            else
                highcount <= 0;
                if lowcount < COUNT_MAX then
                    lowcount  <= lowcount + 1;
                end if;
            end if;
        end if;
    end process;

    process (clk_i) begin
        if clk_i'event and clk_i = '1' then
            if highcount = COUNT_MAX then
                current <= '1';
            elsif lowcount = COUNT_MAX then
                current <= '0';
            end if;
        end if;
    end process;

    rst_o <= current;

end RTL;
